#ifndef _DISPLAY_HPP
#define _DISPLAY_HPP

#include "cloud.hpp"
#include "covariance.hpp"
#include <GL/gl.h>

namespace cloudy
{   
   void display_covariance(const uvector &v,
			   const std::vector<eigen> &eig,
			   double lambda,
			   const color &color1 = Red,
			   const color &color2 = Green,
			   const color &color3 = Blue);
   
   void display_direction(const uvector &v,
			  const uvector &dir,
			  double lambda, 
			  const color &color = Red);
   
   void display_vector(const uvector &v,
		       const uvector &dir,
		       double lambda, 
		       const color &color = Red);

   template <class Cloud1, class Cloud2>
   void display_cloud_directions (const Cloud1 &cl, const Cloud2 &norms,
				  double lambda,
				  const color &color = Red,
				  size_t stride = 1)
   {
      for (size_t i = 0; i < cl.size() && i < norms.size(); i += stride)
	 display_direction(row(cl, i), row(norms, i), lambda, color);
   }

   template <class Cloud1, class Cloud2>
   void display_cloud_vectors (const Cloud1 &cl, const Cloud2 &vs,
			       double lambda,
			       const color &col = Red,
			       size_t stride = 1)
   {
      for (size_t i = 0; i < cl.size() && i < vs.size(); i += stride)
	 display_vector(row(cl, i), row(vs, i), lambda, col);
   }

   template <class Cloud1, class CovarianceVector>
   void display_cloud_covariances (const Cloud1 &cl, const CovarianceVector &covs,
				   double lambda,
				   const color &color1 = Red,
				   const color &color2 = Green,
				   const color &color3 = Blue)
   {
      for (size_t i = 0; i < cl.size() && i < covs.size(); ++i)
	 display_covariance(row(cl, i), covs[i], lambda, color1, color2, color3);
   }

   template <class Cloud>
   void display_cloud (const Cloud &c,
		       const color &col = Black,
		       size_t stride = 1)
   {
      glColor3f(col._r, col._g, col._b);
      glBegin(GL_POINTS);
      for (size_t i = 0; i < c.size(); i += stride)
	 glVertex3f(row(c,i)(0), row(c,i)(1), row(c,i)(2));
      glEnd();
   }

   inline
   void gl_vertex (const uvector &v)
   {
      glVertex3f(v(0), v(1), v(2));
   }

   template <class Cloud, class WeightVector>
   void display_weighted_cloud (const Cloud &c, const WeightVector &v,
				double lambda,
				const color &col = Black)
   {
      glColor3f(col._r, col._g, col._b);
      for (size_t i = 0; i < c.size(); ++i)
      {
	 glPointSize(v[i]*lambda);

	 glBegin(GL_POINTS);
	 glVertex3f(row(c,i)(0), row(c,i)(1), row(c,i)(2));
	 glEnd();
      }
   }
}
#endif
